# -*- coding: utf-8 -*-
# Moovida - Home multimedia server
# Copyright (C) 2006-2009 Fluendo Embedded S.L. (www.fluendo.com).
# All rights reserved.
#
# This file is available under one of two license agreements.
#
# This file is licensed under the GPL version 3.
# See "LICENSE.GPL" in the root of this distribution including a special
# exception to use Moovida with Fluendo's plugins.
#
# The GPL part of Moovida is also available under a commercial licensing
# agreement from Fluendo.
# See "LICENSE.Moovida" in the root directory of this distribution package
# for details on that license.

"""
Unit tests for the notifying dictionary.
"""

from pprint import pformat

from twisted.trial.unittest import TestCase, FailTest

from elisa.core.utils import defer
from elisa.core.utils.notifying_dict import NotifyingDictionary


class TestNotifyingDict(TestCase):

    def setUp(self):
        self.dict = NotifyingDictionary({'one': 1, 'two': 2, 'three': 3})
        self.notifications = []

    def _notify(self, key, value):
        self.notifications.append((key, value))

    def expect_notifications(self, expected):
        dfr = defer.Deferred()
        id = None

        def callback_wrapper(notifier, dict, key, value):
            self._notify(key, value)
            if len(self.notifications) == len(expected):
                self.dict.notifier.disconnect(id)
                if self.notifications == expected:
                    dfr.callback(None)
                else:
                    # Mimic the behaviour of t.t.u._Assertions.failUnlessEqual.
                    dfr.errback(FailTest('not equal:\na = %s\nb = %s\n' % \
                                         (pformat(self.notifications),
                                          pformat(expected))))

        id = self.dict.notifier.connect_after('item-changed', callback_wrapper)
        return dfr

    def test_setitem(self):
        dfr = self.expect_notifications([('two', 5), ('two', 6), ('five', 2)])
        self.dict['two'] = 5
        self.dict['two'] = 6
        self.dict['five'] = 2
        return dfr

    def test_update(self):
        dfr = self.expect_notifications([('six', 6), ('three', 1), ('one', 3)])
        self.dict.update({'one': 3, 'six': 6, 'three': 1})
        return dfr